--         -------------------------------------------
--        | TinyMeleeStats by TheVaan and ArcaneMagus |
--         -------------------------------------------
--
-- File version: 36
-- Project: 36
--

local AceAddon = LibStub("AceAddon-3.0")
local media = LibStub:GetLibrary("LibSharedMedia-3.0")
TinyMeleeStats = AceAddon:NewAddon("TinyMeleeStats", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("TinyMeleeStats")

local ldb = LibStub:GetLibrary("LibDataBroker-1.1");
local TMSBroker = ldb:NewDataObject("TinyMeleeStats", { 
	type = "data source",
	label = "TinyMeleeStats", 
	icon = "Interface\\Icons\\Ability_Racial_ShadowMeld",
	text = "--"
	}
	)
	
TinyMeleeStats.fonts = {}

TinyMeleeStats.defaults = {
	char = {
		Font = "Vera",
		FontEffect = "none",
		Size = 12,
		FrameLocked = true,
		yPosition = 200,
		xPosition = 200,
		inCombatAlpha = 1,
		outOfCombatAlpha = .3,
		RecordMsg = true,
		HighestAp = 0,
		HighestCrit = 0,
		FastestMh = 500,
		FastestOh = 500,
		HighestHit = 0,
		HighestArp = 0,
		Style = {
			Ap = true,
			Crit = true,
			Speed = true,
			Hit = false,
			Arp = false,
			showRecords = true,
			vertical = false,
			labels = false
		},
		Color = {
			ap = {
				r = 1,
				g = 0.803921568627451,
				b = 0
			},
			crit = {
				r = 1,
				g = 0,
				b = 0.6549019607843137
			},
			speed = {
				r = 0,
				g = 0.611764705882353,
				b = 1
			},
			hit = {
				r = 0.07058823529411765,
				g = 0.7686274509803921,
				b = 0
			},
			arp = {
				r = 0,
				g = 0.05490196078431373,
				b = 0.8
			}
		}	
	}
}

TinyMeleeStats.tmsframe = CreateFrame("Frame","TinyMeleeStatsFrame",UIParent)
TinyMeleeStats.tmsframe:SetWidth(100)
TinyMeleeStats.tmsframe:SetHeight(15)
TinyMeleeStats.tmsframe:SetFrameStrata("BACKGROUND")
TinyMeleeStats.tmsframe:EnableMouse(true)
TinyMeleeStats.tmsframe:RegisterForDrag("LeftButton")

TinyMeleeStats.strings = {
	apString = TinyMeleeStats.tmsframe:CreateFontString(),
	critString = TinyMeleeStats.tmsframe:CreateFontString(),
	speedString = TinyMeleeStats.tmsframe:CreateFontString(),
	hitString = TinyMeleeStats.tmsframe:CreateFontString(),
	arpString = TinyMeleeStats.tmsframe:CreateFontString(),

	apRecordString = TinyMeleeStats.tmsframe:CreateFontString(),
	critRecordString = TinyMeleeStats.tmsframe:CreateFontString(),
	speedRecordString = TinyMeleeStats.tmsframe:CreateFontString(),
	hitRecordString = TinyMeleeStats.tmsframe:CreateFontString(),
	arpRecordString = TinyMeleeStats.tmsframe:CreateFontString()
}

function TinyMeleeStats:SetStringColors()
	local c = self.db.char.Color
	self.strings.apString:SetTextColor(c.ap.r, c.ap.g, c.ap.b, 1.0)
	self.strings.critString:SetTextColor(c.crit.r, c.crit.g, c.crit.b, 1.0)
	self.strings.speedString:SetTextColor(c.speed.r, c.speed.g, c.speed.b, 1.0)
	self.strings.hitString:SetTextColor(c.hit.r, c.hit.g, c.hit.b, 1.0)
	self.strings.arpString:SetTextColor(c.arp.r, c.arp.g, c.arp.b, 1.0)
	
	self.strings.apRecordString:SetTextColor(c.ap.r, c.ap.g, c.ap.b, 1.0)
	self.strings.critRecordString:SetTextColor(c.crit.r, c.crit.g, c.crit.b, 1.0)
	self.strings.speedRecordString:SetTextColor(c.speed.r, c.speed.g, c.speed.b, 1.0)
	self.strings.hitRecordString:SetTextColor(c.hit.r, c.hit.g, c.hit.b, 1.0)
	self.strings.arpRecordString:SetTextColor(c.arp.r, c.arp.g, c.arp.b, 1.0)
end

function TinyMeleeStats:OnInitialize()
	local AceConfigReg = LibStub("AceConfigRegistry-3.0")
	local AceConfigDialog = LibStub("AceConfigDialog-3.0")
	
	self.db = LibStub("AceDB-3.0"):New("TinyMeleeStats", TinyMeleeStats.defaults, "char")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("TinyMeleeStats", self:Options(), "tmscmd")
	media.RegisterCallback(self, "LibSharedMedia_Registered")
	
	self:RegisterChatCommand("tms", function() AceConfigDialog:Open("TinyMeleeStats") end)	
	self:RegisterChatCommand("tinymeleestats", function() AceConfigDialog:Open("TinyMeleeStats") end)	
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("TinyMeleeStats", "TinyMeleeStats")
	self.db:RegisterDefaults(self.defaults)
	local version = GetAddOnMetadata("TinyMeleeStats","Version")
	local loaded = L["Open the configuration menu with /tms or /tinymeleestats"].."|r"
	
	DEFAULT_CHAT_FRAME:AddMessage("|cffffd700TinyMeleeStats |cff00ff00~v"..version.."~|cffffd700: "..loaded)
end

function TinyMeleeStats:OnEnable()
	self:LibSharedMedia_Registered()
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnEvent")
	self:RegisterEvent("ADDON_LOADED", "OnEvent")
	self:RegisterEvent("VARIABLES_LOADED", "OnEvent")
	self:RegisterEvent("UNIT_ATTACK_POWER", "OnEvent")
	self:RegisterEvent("UNIT_ATTACK_SPEED", "OnEvent")
	self:RegisterEvent("UNIT_AURA", "OnEvent")
	self:RegisterEvent("UPDATE_SHAPESHIFT_FORM", "OnEvent")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED", "OnEvent")
	self:RegisterEvent("UNIT_LEVEL", "OnEvent")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnEvent")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "OnEvent")
	self:RegisterEvent("PLAYER_TALENT_UPDATE", "OnEvent")
end

function TinyMeleeStats:LibSharedMedia_Registered()
	media:Register("font", "BaarSophia", [[Interface\Addons\TinyMeleeStats\Fonts\BaarSophia.ttf]])
	media:Register("font", "LucidaSD", [[Interface\Addons\TinyMeleeStats\Fonts\LucidaSD.ttf]])
	media:Register("font", "Teen", [[Interface\Addons\TinyMeleeStats\Fonts\Teen.ttf]])
	media:Register("font", "Vera", [[Interface\Addons\TinyMeleeStats\Fonts\Vera.ttf]])
	
	for k, v in pairs(media:List("font")) do
		self.fonts[v] = v
	end
end

function TinyMeleeStats:OnEvent(event, arg1)
	if (event == "ADDON_LOADED") then
		self.tmsframe:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", self.db.char.xPosition, self.db.char.yPosition)
		local font = media:Fetch("font", self.db.char.Font)
		for k, fontObject in pairs(self.strings) do
			fontObject:SetFontObject(GameFontNormal)
			fontObject:SetFont(font, self.db.char.Size, self.db.char.FontEffect)
			fontObject:SetJustifyH("LEFT")
			fontObject:SetJustifyV("MIDDLE")
		end
		self:SetTextAnchors()
		self:SetStringColors()
	end
	if ((event == "PLAYER_REGEN_ENABLED") or (event == "PLAYER_ENTERING_WORLD")) then
		self.tmsframe:SetAlpha(self.db.char.outOfCombatAlpha)
	end
	if (event == "PLAYER_REGEN_DISABLED") then
		self.tmsframe:SetAlpha(self.db.char.inCombatAlpha)
	end
	if (self.db.char.FrameLocked == true) then 
		local fixed = L["Text is fixed. Uncheck Lock Frame in the options to move!"]
		self.tmsframe:SetScript("OnDragStart", function() DEFAULT_CHAT_FRAME:AddMessage(fixed) end)
	end
  	self:Stats()
end

function TinyMeleeStats:SetTextAnchors()
	if (not self.db.char.Style.vertical) then
		self.strings.apString:SetPoint("LEFT", self.tmsframe)
		self.strings.critString:SetPoint("LEFT", self.strings.apString, "RIGHT")
		self.strings.speedString:SetPoint("LEFT", self.strings.critString, "RIGHT")
		self.strings.hitString:SetPoint("LEFT", self.strings.speedString, "RIGHT")
		self.strings.arpString:SetPoint("LEFT", self.strings.hitString, "RIGHT")
		
		self.strings.apRecordString:SetPoint("TOPLEFT", self.strings.apString, "BOTTOMLEFT")
		self.strings.critRecordString:SetPoint("LEFT", self.strings.apRecordString, "RIGHT")
		self.strings.speedRecordString:SetPoint("LEFT", self.strings.critRecordString, "RIGHT")
		self.strings.hitRecordString:SetPoint("LEFT", self.strings.speedRecordString, "RIGHT")
		self.strings.arpRecordString:SetPoint("LEFT", self.strings.hitRecordString, "RIGHT")
	else
		self.strings.apString:SetPoint("LEFT", self.tmsframe)
		self.strings.critString:SetPoint("TOPLEFT", self.strings.apString, "BOTTOMLEFT")
		self.strings.speedString:SetPoint("TOPLEFT", self.strings.critString, "BOTTOMLEFT")
		self.strings.hitString:SetPoint("TOPLEFT", self.strings.speedString, "BOTTOMLEFT")
		self.strings.arpString:SetPoint("TOPLEFT", self.strings.hitString, "BOTTOMLEFT")
		
		self.strings.apRecordString:SetPoint("LEFT", self.strings.apString, "RIGHT")
		self.strings.critRecordString:SetPoint("LEFT", self.strings.critString, "RIGHT")
		self.strings.speedRecordString:SetPoint("LEFT", self.strings.speedString, "RIGHT")
		self.strings.hitRecordString:SetPoint("LEFT", self.strings.hitString, "RIGHT")
		self.strings.arpRecordString:SetPoint("LEFT", self.strings.arpString, "RIGHT")
	end
end

function TinyMeleeStats:Stats()
	local style = self.db.char.Style
	local base, buff, debuff = UnitAttackPower("player")
	local pow = base + buff + debuff
	local crit = string.format("%.2f", GetCritChance("player"))
	local mainSpeed, offSpeed = UnitAttackSpeed("player")
	local speed = 0
	local hit = string.format("%.2f", GetCombatRatingBonus(CR_HIT_MELEE))
	local arp = string.format("%.2f", GetArmorPenetration("player"))
	local recordBroken = "|cffFF0000"..L["Record broken!"]..": "

	if (offSpeed == nil) then
		if (mainSpeed > 0) then
			mainSpeed = string.format("%.2f", mainSpeed)
			speed = mainSpeed
			if (tonumber(mainSpeed) < tonumber(self.db.char.FastestMh)) then self.db.char.FastestMh = mainSpeed end
			fastestSpeed = self.db.char.FastestMh
		else
			speed = 0
			mainSpeed = 0
		end
	else
		if (mainSpeed > 0) then
			mainSpeed = string.format("%.2f", mainSpeed)
			offSpeed = string.format("%.2f", offSpeed)
			speed = mainSpeed.."s "..offSpeed
			if (tonumber(mainSpeed) < tonumber(self.db.char.FastestMh)) then self.db.char.FastestMh = mainSpeed end
			if (tonumber(offSpeed) < tonumber(self.db.char.FastestOh)) then self.db.char.FastestOh = offSpeed end
			fastestSpeed = self.db.char.FastestMh.."s "..self.db.char.FastestOh
		else
			speed = 0
			mainSpeed = 0
			offSpeed = 0
			fastestSpeed = 0
		end
	end
	
	if (tonumber(pow) > tonumber(self.db.char.HighestAp)) then
		self.db.char.HighestAp = pow
		if (self.db.char.RecordMsg == true) then
			DEFAULT_CHAT_FRAME:AddMessage(recordBroken.."|cffFF0000"..L["Attack Power"]..": |c00ffef00"..self.db.char.HighestAp.."|r")
		end
	end
	if (tonumber(crit) > tonumber(self.db.char.HighestCrit)) then
		self.db.char.HighestCrit = crit
		if (self.db.char.RecordMsg == true) then
			DEFAULT_CHAT_FRAME:AddMessage(recordBroken.."|cffFF0000"..L["Critical Chance"]..": |c00ffef00"..self.db.char.HighestCrit.."|r")
		end
	end
	if (tonumber(hit) > tonumber(self.db.char.HighestHit)) then
		self.db.char.HighestHit = hit
		if (self.db.char.RecordMsg == true) then
			DEFAULT_CHAT_FRAME:AddMessage(recordBroken.."|cffFF0000"..L["Hit rating"]..": |c00ffef00"..self.db.char.HighestHit.."|r")
		end
	end
	if (tonumber(arp) > tonumber(self.db.char.HighestArp)) then
		self.db.char.HighestArp = arp
		if (self.db.char.RecordMsg == true) then
			DEFAULT_CHAT_FRAME:AddMessage(recordBroken.."|cffFF0000"..L["Armor Penetration"]..": |c00ffef00"..self.db.char.HighestArp.."|r")
		end
	end
	
	if (style.Ap == true) then
		local apTempString = ""
		local apRecordTempString = ""
		if (style.labels) then
			apTempString = apTempString..L["Ap:"].." "
		end
		apTempString = apTempString..pow
		if (style.showRecords) then
			if (style.labels) then
				apRecordTempString = apRecordTempString..L["Ap:"].." "
			end
			apRecordTempString = apRecordTempString..self.db.char.HighestAp
		end
		self.strings.apString:SetText(apTempString)
		self.strings.apRecordString:SetText(apRecordTempString)
	else
		self.strings.apString:SetText("")
		self.strings.apRecordString:SetText("")
	end
	if (style.Crit == true) then
		local critTempString = ""
		local critRecordTempString = ""
		if (style.labels) then
			critTempString = critTempString..L["Crit:"].." "
		end
		critTempString = critTempString..crit
		if (style.showRecords) then
			if (style.labels) then
				critRecordTempString = critRecordTempString..L["Crit:"].." "
			end
			critRecordTempString = critRecordTempString..self.db.char.HighestCrit
		end
		self.strings.critString:SetText(critTempString)
		self.strings.critRecordString:SetText(critRecordTempString)
	else
		self.strings.critString:SetText("")
		self.strings.critRecordString:SetText("")
	end
	if (style.Hit == true) then
		local hitTempString = ""
		local hitRecordTempString = ""
		if (style.labels) then
			hitTempString = hitTempString..L["Hit:"].." "
		end
		hitTempString = hitTempString..hit
		if (style.showRecords) then
			if (style.labels) then
				hitRecordTempString = hitRecordTempString..L["Hit:"].." "
			end
			hitRecordTempString = hitRecordTempString..string.format("%.2f", self.db.char.HighestHit)
		end
		self.strings.hitString:SetText(hitTempString)
		self.strings.hitRecordString:SetText(hitRecordTempString)
	else
		self.strings.hitString:SetText("")
		self.strings.hitRecordString:SetText("")
	end
	if (style.Speed == true) then
		local speedTempString = ""
		local speedRecordTempString = ""
		if (style.labels) then
			speedTempString = speedTempString..L["Speed:"].." "
		end
		speedTempString = speedTempString..speed.."s"
		if (style.showRecords) then
			if (style.labels) then
				speedRecordTempString = speedRecordTempString..L["Speed:"].." "
			end
			speedRecordTempString = speedRecordTempString..fastestSpeed.."s"
		end
		self.strings.speedString:SetText(speedTempString)
		self.strings.speedRecordString:SetText(speedRecordTempString)
	else
		self.strings.speedString:SetText("")
		self.strings.speedRecordString:SetText("")
	end
	if (style.Arp == true) then
		local arpTempString = ""
		local arpRecordTempString = ""
		if (style.labels) then
			arpTempString = arpTempString..L["Arp:"].." "
		end
		arpTempString = arpTempString..arp
		if (style.showRecords) then
			if (style.labels) then
				arpRecordTempString = arpRecordTempString..L["Arp:"].." "
			end
			arpRecordTempString = arpRecordTempString..string.format("%.2f", self.db.char.HighestArp)
		end
		self.strings.arpString:SetText(arpTempString)
		self.strings.arpRecordString:SetText(arpRecordTempString)
	else
		self.strings.arpString:SetText("")
		self.strings.arpRecordString:SetText("")
	end
	
	--TMSBroker.text = stat
end
